#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */

#include "peri.h"
#include "usbdrv.h"

#include <stdio.h>
#include <unistd.h>
#include <tm1637.h>

#define RQ_SET_LED         0
#define RQ_SET_LED_VALUE   1
#define RQ_GET_SWITCH      2
#define RQ_GET_LIGHT       3
#define RQ_SET_LED2        4
#define RQ_SET_LED_VALUE2  5

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    /* declared as static so they stay valid when usbFunctionSetup returns */
    static uint8_t switch_state;
    static uint16_t dark;  

    if (rq->bRequest == RQ_SET_LED)
    {
        uint8_t led_state = rq->wValue.bytes[0];
        uint8_t led_no  = rq->wIndex.bytes[0];
        set_led(led_no, led_state);
        return 0;
    }
   /* else if (rq->bRequest == RQ_SET_LED2)
    {
        uint8_t led_state2 = rq->wValue.bytes[0];
        uint8_t led_no2  = rq->wIndex.bytes[0];
        set_led2((led_no2, led_state2);
        return 0;
    }*/

    else if (rq->bRequest == RQ_GET_SWITCH)
    {
        switch_state = SWITCH_PRESSED();

        // point usbMsgPtr to the data to be returned to host
        usbMsgPtr = &switch_state;

        // return the number of bytes of data to be returned to host
        return 1;
    }

    else if (rq->bRequest == RQ_SET_LED_VALUE)
    {
	uint8_t num = rq->wValue.bytes[0];
	set_led_value(num);
	return 0;
    }
   /* else if (rq->bRequest == RQ_SET_LED_VALUE2)
    {
        uint8_t num2 = rq->wValue.bytes[0];
        set_led_value(num2);
        return 0;
    }*/
    else if (rq->bRequest == RQ_GET_LIGHT)
    {
	dark = read_adc(PC4);
	usbMsgPtr = &dark;
	return 2;
    }
/*
    else if (rq->bRequest == set_digit)
    {
        dark = read_adc(PC4);
        usbMsgPtr = &dark;
        return 2;
    }
    // default for not implemented requests: return no data back to host 
    return 0;
}
*/
/* ------------------------------------------------------------------------- */
int main(void)
{
    init_peri();
    //init_peri2();

    usbInit();
]
    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(1000);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */

